local T, C, L, _ = unpack(select(2, ...))
if C.bag.enable ~= true then return end

------------------------------------------------------------------------------------
-- Credit: tdBag2
------------------------------------------------------------------------------------
local bankFrame = CreateFrame("Frame", "OfflineBag", UIParent)
bankFrame:EnableMouse(true)
bankFrame:SetMovable(true)
bankFrame:SetClampedToScreen(true)
bankFrame:RegisterForDrag("LeftButton")
bankFrame:SetScript("OnDragStart", function(self) self:StartMoving() end)
bankFrame:SetScript("OnDragStop", function(self) self:StopMovingOrSizing() end)

local BANKS = {-1, 5, 6, 7, 8, 9, 10, 11}
local atBank
local ItemButtons = {}
local dataCache = {}

local function NameGenerator(name)
	local index = 0
	return function()
		index = index + 1
		return name .. index
	end
end

local GenerateName = NameGenerator("OfflineBagSlot")

local function SetupCache()
	ShestakUIOfflineBag = ShestakUIOfflineBag or {}
	PlayerDB = ShestakUIOfflineBag

	for _, bag in ipairs(BANKS) do
		ItemButtons[bag] = {}
	end
end

local function getInfo(itemData)
	if not dataCache[itemData] then
		local data = {}
		if itemData then
			data.id = tonumber(itemData[1])
			data.count = tonumber(itemData[2])
			data.icon = GetItemIcon(data.id)
			
			local name, link, quality = GetItemInfo(data.id)
			data.link = link
			data.name = name
			data.quality = tonumber(quality)
		end

		dataCache[itemData] = data
	end

	return dataCache[itemData]
end

local function SaveBag(bag)
	local size = GetContainerNumSlots(bag)
	local items
	if size > 0 then
		items = {}
		items.size = size

		for slot = 1, size do
			local icon, itemCount, _, quality, _, _, itemLink, _, _, itemID = GetContainerItemInfo(bag, slot)
			if itemID then
				items[slot] = {itemID, itemCount}
			end
		end
	end
	PlayerDB[bag] = items
	ShestakUIOfflineBag = PlayerDB
end

local function Bank_Opened()
	atBank = true
end

local function Bank_Closed()
	if atBank then
		for _, bag in ipairs(BANKS) do
			SaveBag(bag)
		end
		atBank = nil
	end
end

local function ItemButton_OnEnter(self)
	if self.info and self.info.link then
		GameTooltip:SetOwner(self, 'ANCHOR_RIGHT')
		GameTooltip:SetHyperlink(self.info.link)
		GameTooltip:Show()
	end
end

local function HideTooltip()
	GameTooltip:Hide()
end

local function ItemButton_UpdateBorder(self)
	if not self.info then return end
	local classID = select(12, GetItemInfo(self.info.id))
	if self.info.quality and self.info.quality > 1 and classID ~= 12 then
		local r, g, b = GetItemQualityColor(self.info.quality)
		self:SetBackdropBorderColor(r, g, b)
	elseif classID == 12 then
		self:SetBackdropBorderColor(1, 1, 0)
	else
		self:SetBackdropBorderColor(0.37, 0.3, 0.3, 1)
	end
end

local function ItemButton_UpdateItem(self)
	if not self.info then return end

	self.Icon:SetTexture(self.info.icon)

	if self.info.count and self.info.count > 1 then
		self.Count:SetText(self.info.count)
		self.Count:Show()
	else
		self.Count:Hide()
	end
end

local function ItemButton_LoadInfo(self, itemData)
	self.info = getInfo(itemData)
end

local function ItemButton_Update(self, itemData)
	ItemButton_LoadInfo(self, itemData)
	ItemButton_UpdateItem(self)
	ItemButton_UpdateBorder(self)
end

local function ItemButton_UpdateInfo(self)
	local name, link, quality = GetItemInfo(self.info.id)
	if name then
		self.info.name = name
		self.info.link = link
		self.info.quality = quality
	end

	ItemButton_UpdateBorder(self)
end

local function ItemButton_Free(self)
	self:Hide()
	self:SetBackdropColor(0, 0, 0)
	self.Icon:SetTexture("")
	self.info = nil
	self.Count:SetText("")
	self.Count:Hide()
	self:SetBackdropBorderColor(0.37, 0.3, 0.3, 1)
end

local function UpdateItemButtonInfo(_, itemId)
	for bag, buttons in pairs(ItemButtons) do
		for slot, itemButton in pairs(buttons) do
			if itemButton:IsShown() and itemButton.info and itemButton.info.id and itemButton.info.id == itemId then
				itemButton:UpdateInfo()
			end
		end
	end
end

local function CreateItemButton(bag, slot)
	local name = GenerateName()
	local button = CreateFrame("Button", name, nil, "ItemButtonTemplate")
	local iconSize = C.bag.button_size

	button:StripTextures()
	button:SetNormalTexture("")
	button:SetPushedTexture("")
	button:SetHighlightTexture("Interface\\ChatFrame\\ChatFrameBackground")
	button:GetHighlightTexture():SetVertexColor(1, 1, 1, .3)
	button:SetSize(iconSize, iconSize)
	button:Hide()

	button.Icon = _G[name.."IconTexture"]
	button.Icon:SetTexCoord(0.1, 0.9, 0.1, 0.9)
	button.Icon:SetPoint("TOPLEFT", button, 2, -2)
	button.Icon:SetPoint("BOTTOMRIGHT", button, -2, 2)
	button.Count = _G[name.."Count"]
	button.Count:SetPoint("BOTTOMRIGHT", -1, 2)
	button.Count:SetFont(C.font.bags_font, C.font.bags_font_size, C.font.bags_font_style)
	button.bagID = bag
	button.slotID = slot

	Mixin(button, BackdropTemplateMixin)
	button:OnBackdropLoaded()
	button:HookScript("OnSizeChanged", button.OnBackdropSizeChanged)
	button:SetBackdrop({
		bgFile = C.media.blank, edgeFile = C.media.blank, edgeSize = T.mult,
		insets = {left = -T.mult, right = -T.mult, top = -T.mult, bottom = -T.mult}
	})
	button:SetBackdropColor(0.37, 0.3, 0.3, 1)

	button:SetScript("OnEnter", ItemButton_OnEnter)
	button:SetScript("OnLeave", HideTooltip)

	button.Update = ItemButton_Update
	button.UpdateInfo = ItemButton_UpdateInfo
	button.Free = ItemButton_Free

	ItemButtons[bag][slot] = button

	return button
end

local function GetItemButton(bag, slot)
	return ItemButtons[bag][slot] or CreateItemButton(bag, slot)
end

local function FreeAllButtons()
	for bag, buttons in pairs(ItemButtons) do
		for slot, itemButton in pairs(buttons) do
			itemButton:Free()
		end
	end
end

local function LayoutButtons(frame, bags)
	local ownerDB = PlayerDB
	local x, y = 0, 0
	local column = C.bag.bank_columns
	local offset = 38
	local spacing = C.bag.button_space
	local iconSize = C.bag.button_size
	local size = iconSize + spacing
	local xOffset = 5
	local yOffset = -offset + xOffset

	for _, bag in ipairs(bags) do
		local bagData = ownerDB[bag]
		if bagData and bagData.size and bagData.size > 0 then
			for slot = 1, bagData.size do
				local itemButton = GetItemButton(bag, slot)
				local itemData = bagData[slot]
				if x == column then
					y = y + 1
					x = 0
				end
				itemButton:SetParent(frame)
				itemButton:ClearAllPoints()
				itemButton:SetPoint('TOPLEFT', frame, 'TOPLEFT', x * size + xOffset , -y * size + yOffset)
				itemButton:Show()

				if itemData then
					itemButton:Update(itemData)
					itemButton:UpdateInfo(itemData)
				end

				x = x + 1
			end
		end
	end

	local width = column * (iconSize + spacing) - spacing
	local height = (y + 1) * (iconSize + spacing) - spacing
	frame:SetSize(width + xOffset * 2, height + offset)
end

local function UpdateAllBags()
	FreeAllButtons()
	LayoutButtons(bankFrame, BANKS)
end

local function CreateMainFrame()
	bankFrame:SetPoint("TOPLEFT", UIParent, "LEFT", 150, 50)
	bankFrame:SetTemplate("Transparent")
	bankFrame:SetFrameStrata("HIGH")
	bankFrame:SetClampedToScreen(true)
	bankFrame:Hide()
	bankFrame:SetScript("OnShow", function() UpdateAllBags() end)
	bankFrame:FontString("text", C.font.bags_font, C.font.bags_font_size, C.font.bags_font_style)
	bankFrame.text:SetPoint("TOP", 0, -4)
	bankFrame.text:SetText(L_BAG_OfflineBag)

	local Close = CreateFrame("Button", "OfflineBagCloseButton", bankFrame, "UIPanelCloseButton")
	T.SkinCloseButton(Close, nil, nil, true)
	Close:SetSize(16, 16)
	Close:RegisterForClicks("AnyUp")
	Close:SetScript("OnClick", function()
		bankFrame:Hide()
		PlaySound(SOUNDKIT.IG_BACKPACK_CLOSE)
	end)
end

local function OnLogin()
	SetupCache()
	UpdateAllBags()
	CreateMainFrame()
	T.UnregisterEvent("ADDON_LOADED", OnLogin)
end

T.RegisterEvent("ADDON_LOADED", OnLogin)
T.RegisterEvent("BANKFRAME_OPENED", Bank_Opened)
T.RegisterEvent("BANKFRAME_CLOSED", Bank_Closed)
T.RegisterEvent("GET_ITEM_INFO_RECEIVED", UpdateItemButtonInfo)
